/* ========================================================================= *
 * Array generator
 * Implementation of the Array generator interface
 * ========================================================================= */

#include <stdlib.h>
#include <stddef.h>
#include <stdbool.h>
#include <math.h>  

#include "Array.h"

/* ------------------------------------------------------------------------- *
 * Create a sorted array of integers (from 0 to length-1).
 *
 * The array must later be deleted by calling free().
 *
 * PARAMETERS
 * length       Number of elements in the array (pre-condition: 0 < length)
 *
 * RETURN
 * array        A new array of integers, or NULL in case of error
 * ------------------------------------------------------------------------- */
static int* createSortedArray(size_t length)
{
    if (length == 0)
      return NULL;

    int* array = (int*) malloc(length * sizeof(int));

    if (array == NULL)
        return NULL;


    for (size_t i = 0; i < length; i++)
        array[i] = i;

    return array;
}

int* createNearlySortedArray(size_t length)
{
    int* array = createSortedArray(length);
    if (array == NULL)
        return NULL;

    size_t nbSwap = (size_t)(log2((double)length));
    for(size_t i=0; i < nbSwap; i++)
    {
        size_t origin = rand() % length;
        size_t dest = rand() % length;
        int tmp = array[origin];
        array[origin] = array[dest];
        array[dest] = tmp;
    }

    return array;
}


int* createRandomArray(size_t length)
{

    int* array = createSortedArray(length);
    if (array == NULL)
        return NULL;


    // Fisher-Yates shuffling algorithm
    size_t i = length - 1;
    size_t j;
    int temp;

    while (i > 0)
    {
        j = (size_t)rand() % (i + 1);
        temp = array[i];
        array[i] = array[j];
        array[j] = temp;
        i--;
    }

    return array;
}


int* createFewUniquesArray(size_t length)
{
    if (length == 0)
        return NULL;

    int* array = (int*) malloc(length * sizeof(int));

    if (array == NULL)
        return NULL;


    size_t nbUniques = (size_t)(log2((double)length)+1.5);

    for (size_t i = 0; i < length; i++)
        array[i] = rand() % nbUniques;

    return array;
}
